#define _MATLAB
#ifdef _MATLAB
#include "mex.h"
#include "image.h"
#include "image3.h"
#include "misc.h"
// Convert mex array to image<>
// Convert image<> to mex array

static char errMsg[256];

image<uchar> *label_scale(image3<float> *dataTerm, double disc_k, double data_k, double lambda);

image3<float>* load_matlab_matrix(const mxArray *mxImage)
{
	mxAssert(mxIsClass(mxImage, "double"), "Input should be double.");
	int nDim = mxGetNumberOfDimensions(mxImage);
	const int* imDim = mxGetDimensions(mxImage);

	int imWidth = imDim[1], imHeight = imDim[0], channel;
	if (nDim == 3) {
		channel = imDim[2];
	}
	else if (nDim == 2) {
		channel = 1;
	}
	else
	{
		sprintf(errMsg, "Dimension of input data term is %d, unsupported.", nDim);
		mexErrMsgTxt(errMsg);
	}


#ifdef DEBUG
	mexPrintf("%s\n", mxGetClassName(mxImage));
    mexPrintf("w:%d h:%d n:%d\n", imWidth, imHeight, channel);
#endif

	image3<float> *mat = new image3<float>(imWidth, imHeight, channel);

	double *pMatlabPlane = (double*)mxGetData(mxImage);

	int nPixels = imHeight * imWidth;
	for (int i = 0; i < imHeight; i++)
		for (int j = 0; j < imWidth; j++)
			for (int k = 0; k < channel; k++)
				imRef3(mat, j, i, k) = pMatlabPlane[k*nPixels+j*imHeight+i]; 

	return mat;
}

mxArray* convert_to_matlab(image<uchar> *label_img)
{
	int dims[2];
	int height = label_img->height(), width = label_img->width();
	dims[0] = height, dims[1] = width;
	mxArray *matrix = mxCreateNumericArray(2, dims, mxDOUBLE_CLASS, mxREAL);
	double *pMatlabPlane = (double*)mxGetData(matrix);
	for (int i=0; i < height; i++)
		for (int j = 0; j < width; j++)
			pMatlabPlane[j*height+i] = imRef(label_img, j, i);
	return matrix;	
}

// Load parameters from mex function input.
void load_parameter(const mxArray *para_array, double& disc_k, double& data_k, double& lambda)
{
	int nDims = mxGetNumberOfDimensions(para_array);
	const char* para_msg = "Parameter should be a vector.";
	if (nDims > 2) mexErrMsgTxt(para_msg);
	const int* dims = mxGetDimensions(para_array);
	if (dims[0] != 1 && dims[1] != 1) mexErrMsgTxt(para_msg);
	mxAssert((dims[0] * dims[1] == 3), "Number of parameters not correct"); 
	
	double *para = (double*)mxGetData(para_array);
	disc_k = para[0];
	data_k = para[1];
	lambda = para[2];
	return;
}

// input: data term and parameters
// First parameter is data term
// define energy function in code
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	if (nrhs < 1)
		mexErrMsgTxt("Input data term for optimization.");
	if (nrhs < 2)
		mexErrMsgTxt("Input optimization parameters.");
	if (nlhs != 1)
		mexErrMsgTxt("Output not specified for result.");

	// load parameters for matching.
	double disc_k = 0, data_k = 0, lambda = 0;
	// disc_k : truncation of discontinuity cost
	// data_k : truncation of data cost
	// lambda : weight of data cost
	load_parameter(prhs[1], disc_k, data_k, lambda);	
		
	const int* dims = mxGetDimensions(prhs[0]);

	image3<float> *dataTerm = load_matlab_matrix(prhs[0]);
	image<uchar> *scaleLabel = label_scale(dataTerm, disc_k, data_k, lambda);

	plhs[0] = convert_to_matlab(scaleLabel);
}
#endif
